#include "ellLib.h"
#include "epicsMutex.h"
#include "link.h"
#include "epicsTime.h"
#include "epicsTypes.h"

#ifndef INCabDcm_DCMTH
#define INCabDcm_DCMTH
typedef enum {
	abDcm_1771DCM,
	abDcm_1771PLCadapter,
	abDcm_1771MultiSlot
}abDcm_DCMT;
#endif /*INCabDcm_DCMTH*/
#ifndef INCabDcmH
#define INCabDcmH
typedef struct abDcmRecord {
	char		name[61]; /*Record Name*/
	char		desc[29]; /*Descriptor*/
	char		asg[29]; /*Access Security Group*/
	epicsEnum16	scan;	/*Scan Mechanism*/
	epicsEnum16	pini;	/*Process at iocInit*/
	short		phas;	/*Scan Phase*/
	short		evnt;	/*Event Number*/
	short		tse;	/*Time Stamp Event*/
	DBLINK		tsel;	/*Time Stamp Link*/
	epicsEnum16	dtyp;	/*Device Type*/
	short		disv;	/*Disable Value*/
	short		disa;	/*Disable*/
	DBLINK		sdis;	/*Scanning Disable*/
	epicsMutexId	mlok;	/*Monitor lock*/
	ELLLIST		mlis;	/*Monitor List*/
	unsigned char	disp;	/*Disable putField*/
	unsigned char	proc;	/*Force Processing*/
	epicsEnum16	stat;	/*Alarm Status*/
	epicsEnum16	sevr;	/*Alarm Severity*/
	epicsEnum16	nsta;	/*New Alarm Status*/
	epicsEnum16	nsev;	/*New Alarm Severity*/
	epicsEnum16	acks;	/*Alarm Ack Severity*/
	epicsEnum16	ackt;	/*Alarm Ack Transient*/
	epicsEnum16	diss;	/*Disable Alarm Sevrty*/
	unsigned char	lcnt;	/*Lock Count*/
	unsigned char	pact;	/*Record active*/
	unsigned char	putf;	/*dbPutField process*/
	unsigned char	rpro;	/*Reprocess */
	void		*asp;	/*Access Security Pvt*/
	struct putNotify *ppn;	/*addr of PUTNOTIFY*/
	struct putNotifyRecord *ppnr;	/*pputNotifyRecord*/
	struct scan_element *spvt;	/*Scan Private*/
	struct rset	*rset;	/*Address of RSET*/
	struct dset	*dset;	/*DSET address*/
	void		*dpvt;	/*Device Private*/
	struct dbRecordType *rdes;	/*Address of dbRecordType*/
	struct lockRecord *lset;	/*Lock Set*/
	epicsEnum16	prio;	/*Scheduling Priority*/
	unsigned char	tpro;	/*Trace Processing*/
	char bkpt;	/*Break Point*/
	unsigned char	udf;	/*Undefined*/
	epicsTimeStamp	time;	/*Time*/
	DBLINK		flnk;	/*Forward Process Link*/
	char		val[40]; /*Link Status*/
	char		oval[40]; /*Link Status*/
	epicsEnum16	dcmt;	/*DCM Type*/
	unsigned short	nt;	/*Number of Tables*/
	unsigned short	toff;	/*Offset in message of Table Number*/
	float		dly;	/*block transfer delay */
	epicsEnum16	loca;	/*scanIoRequest comm err*/
	unsigned short	link;	/*Scanner Number*/
	unsigned short	rack;	/*Rack*/
	unsigned short	slot;	/*Slot*/
	unsigned short	mcw;	/*Max Consecutive Writes*/
	void *		t0;	/*Table Array 0*/
	void *		t1;	/*Table Array 1*/
	void *		t2;	/*Table Array 2*/
	void *		t3;	/*Table Array 3*/
	void *		t4;	/*Table Array 4*/
	void *		t5;	/*Table Array 5*/
	void *		t6;	/*Table Array 6*/
	void *		t7;	/*Table Array 7*/
	void *		t8;	/*Table Array 8*/
	void *		t9;	/*Table Array 9*/
} abDcmRecord;
#define abDcmRecordNAME	0
#define abDcmRecordDESC	1
#define abDcmRecordASG	2
#define abDcmRecordSCAN	3
#define abDcmRecordPINI	4
#define abDcmRecordPHAS	5
#define abDcmRecordEVNT	6
#define abDcmRecordTSE	7
#define abDcmRecordTSEL	8
#define abDcmRecordDTYP	9
#define abDcmRecordDISV	10
#define abDcmRecordDISA	11
#define abDcmRecordSDIS	12
#define abDcmRecordMLOK	13
#define abDcmRecordMLIS	14
#define abDcmRecordDISP	15
#define abDcmRecordPROC	16
#define abDcmRecordSTAT	17
#define abDcmRecordSEVR	18
#define abDcmRecordNSTA	19
#define abDcmRecordNSEV	20
#define abDcmRecordACKS	21
#define abDcmRecordACKT	22
#define abDcmRecordDISS	23
#define abDcmRecordLCNT	24
#define abDcmRecordPACT	25
#define abDcmRecordPUTF	26
#define abDcmRecordRPRO	27
#define abDcmRecordASP	28
#define abDcmRecordPPN	29
#define abDcmRecordPPNR	30
#define abDcmRecordSPVT	31
#define abDcmRecordRSET	32
#define abDcmRecordDSET	33
#define abDcmRecordDPVT	34
#define abDcmRecordRDES	35
#define abDcmRecordLSET	36
#define abDcmRecordPRIO	37
#define abDcmRecordTPRO	38
#define abDcmRecordBKPT	39
#define abDcmRecordUDF	40
#define abDcmRecordTIME	41
#define abDcmRecordFLNK	42
#define abDcmRecordVAL	43
#define abDcmRecordOVAL	44
#define abDcmRecordDCMT	45
#define abDcmRecordNT	46
#define abDcmRecordTOFF	47
#define abDcmRecordDLY	48
#define abDcmRecordLOCA	49
#define abDcmRecordLINK	50
#define abDcmRecordRACK	51
#define abDcmRecordSLOT	52
#define abDcmRecordMCW	53
#define abDcmRecordT0	54
#define abDcmRecordT1	55
#define abDcmRecordT2	56
#define abDcmRecordT3	57
#define abDcmRecordT4	58
#define abDcmRecordT5	59
#define abDcmRecordT6	60
#define abDcmRecordT7	61
#define abDcmRecordT8	62
#define abDcmRecordT9	63
#endif /*INCabDcmH*/
#ifdef GEN_SIZE_OFFSET
#ifdef __cplusplus
extern "C" {
#endif
#include <epicsExport.h>
static int abDcmRecordSizeOffset(dbRecordType *pdbRecordType)
{
    abDcmRecord *prec = 0;
  pdbRecordType->papFldDes[0]->size=sizeof(prec->name);
  pdbRecordType->papFldDes[0]->offset=(short)((char *)&prec->name - (char *)prec);
  pdbRecordType->papFldDes[1]->size=sizeof(prec->desc);
  pdbRecordType->papFldDes[1]->offset=(short)((char *)&prec->desc - (char *)prec);
  pdbRecordType->papFldDes[2]->size=sizeof(prec->asg);
  pdbRecordType->papFldDes[2]->offset=(short)((char *)&prec->asg - (char *)prec);
  pdbRecordType->papFldDes[3]->size=sizeof(prec->scan);
  pdbRecordType->papFldDes[3]->offset=(short)((char *)&prec->scan - (char *)prec);
  pdbRecordType->papFldDes[4]->size=sizeof(prec->pini);
  pdbRecordType->papFldDes[4]->offset=(short)((char *)&prec->pini - (char *)prec);
  pdbRecordType->papFldDes[5]->size=sizeof(prec->phas);
  pdbRecordType->papFldDes[5]->offset=(short)((char *)&prec->phas - (char *)prec);
  pdbRecordType->papFldDes[6]->size=sizeof(prec->evnt);
  pdbRecordType->papFldDes[6]->offset=(short)((char *)&prec->evnt - (char *)prec);
  pdbRecordType->papFldDes[7]->size=sizeof(prec->tse);
  pdbRecordType->papFldDes[7]->offset=(short)((char *)&prec->tse - (char *)prec);
  pdbRecordType->papFldDes[8]->size=sizeof(prec->tsel);
  pdbRecordType->papFldDes[8]->offset=(short)((char *)&prec->tsel - (char *)prec);
  pdbRecordType->papFldDes[9]->size=sizeof(prec->dtyp);
  pdbRecordType->papFldDes[9]->offset=(short)((char *)&prec->dtyp - (char *)prec);
  pdbRecordType->papFldDes[10]->size=sizeof(prec->disv);
  pdbRecordType->papFldDes[10]->offset=(short)((char *)&prec->disv - (char *)prec);
  pdbRecordType->papFldDes[11]->size=sizeof(prec->disa);
  pdbRecordType->papFldDes[11]->offset=(short)((char *)&prec->disa - (char *)prec);
  pdbRecordType->papFldDes[12]->size=sizeof(prec->sdis);
  pdbRecordType->papFldDes[12]->offset=(short)((char *)&prec->sdis - (char *)prec);
  pdbRecordType->papFldDes[13]->size=sizeof(prec->mlok);
  pdbRecordType->papFldDes[13]->offset=(short)((char *)&prec->mlok - (char *)prec);
  pdbRecordType->papFldDes[14]->size=sizeof(prec->mlis);
  pdbRecordType->papFldDes[14]->offset=(short)((char *)&prec->mlis - (char *)prec);
  pdbRecordType->papFldDes[15]->size=sizeof(prec->disp);
  pdbRecordType->papFldDes[15]->offset=(short)((char *)&prec->disp - (char *)prec);
  pdbRecordType->papFldDes[16]->size=sizeof(prec->proc);
  pdbRecordType->papFldDes[16]->offset=(short)((char *)&prec->proc - (char *)prec);
  pdbRecordType->papFldDes[17]->size=sizeof(prec->stat);
  pdbRecordType->papFldDes[17]->offset=(short)((char *)&prec->stat - (char *)prec);
  pdbRecordType->papFldDes[18]->size=sizeof(prec->sevr);
  pdbRecordType->papFldDes[18]->offset=(short)((char *)&prec->sevr - (char *)prec);
  pdbRecordType->papFldDes[19]->size=sizeof(prec->nsta);
  pdbRecordType->papFldDes[19]->offset=(short)((char *)&prec->nsta - (char *)prec);
  pdbRecordType->papFldDes[20]->size=sizeof(prec->nsev);
  pdbRecordType->papFldDes[20]->offset=(short)((char *)&prec->nsev - (char *)prec);
  pdbRecordType->papFldDes[21]->size=sizeof(prec->acks);
  pdbRecordType->papFldDes[21]->offset=(short)((char *)&prec->acks - (char *)prec);
  pdbRecordType->papFldDes[22]->size=sizeof(prec->ackt);
  pdbRecordType->papFldDes[22]->offset=(short)((char *)&prec->ackt - (char *)prec);
  pdbRecordType->papFldDes[23]->size=sizeof(prec->diss);
  pdbRecordType->papFldDes[23]->offset=(short)((char *)&prec->diss - (char *)prec);
  pdbRecordType->papFldDes[24]->size=sizeof(prec->lcnt);
  pdbRecordType->papFldDes[24]->offset=(short)((char *)&prec->lcnt - (char *)prec);
  pdbRecordType->papFldDes[25]->size=sizeof(prec->pact);
  pdbRecordType->papFldDes[25]->offset=(short)((char *)&prec->pact - (char *)prec);
  pdbRecordType->papFldDes[26]->size=sizeof(prec->putf);
  pdbRecordType->papFldDes[26]->offset=(short)((char *)&prec->putf - (char *)prec);
  pdbRecordType->papFldDes[27]->size=sizeof(prec->rpro);
  pdbRecordType->papFldDes[27]->offset=(short)((char *)&prec->rpro - (char *)prec);
  pdbRecordType->papFldDes[28]->size=sizeof(prec->asp);
  pdbRecordType->papFldDes[28]->offset=(short)((char *)&prec->asp - (char *)prec);
  pdbRecordType->papFldDes[29]->size=sizeof(prec->ppn);
  pdbRecordType->papFldDes[29]->offset=(short)((char *)&prec->ppn - (char *)prec);
  pdbRecordType->papFldDes[30]->size=sizeof(prec->ppnr);
  pdbRecordType->papFldDes[30]->offset=(short)((char *)&prec->ppnr - (char *)prec);
  pdbRecordType->papFldDes[31]->size=sizeof(prec->spvt);
  pdbRecordType->papFldDes[31]->offset=(short)((char *)&prec->spvt - (char *)prec);
  pdbRecordType->papFldDes[32]->size=sizeof(prec->rset);
  pdbRecordType->papFldDes[32]->offset=(short)((char *)&prec->rset - (char *)prec);
  pdbRecordType->papFldDes[33]->size=sizeof(prec->dset);
  pdbRecordType->papFldDes[33]->offset=(short)((char *)&prec->dset - (char *)prec);
  pdbRecordType->papFldDes[34]->size=sizeof(prec->dpvt);
  pdbRecordType->papFldDes[34]->offset=(short)((char *)&prec->dpvt - (char *)prec);
  pdbRecordType->papFldDes[35]->size=sizeof(prec->rdes);
  pdbRecordType->papFldDes[35]->offset=(short)((char *)&prec->rdes - (char *)prec);
  pdbRecordType->papFldDes[36]->size=sizeof(prec->lset);
  pdbRecordType->papFldDes[36]->offset=(short)((char *)&prec->lset - (char *)prec);
  pdbRecordType->papFldDes[37]->size=sizeof(prec->prio);
  pdbRecordType->papFldDes[37]->offset=(short)((char *)&prec->prio - (char *)prec);
  pdbRecordType->papFldDes[38]->size=sizeof(prec->tpro);
  pdbRecordType->papFldDes[38]->offset=(short)((char *)&prec->tpro - (char *)prec);
  pdbRecordType->papFldDes[39]->size=sizeof(prec->bkpt);
  pdbRecordType->papFldDes[39]->offset=(short)((char *)&prec->bkpt - (char *)prec);
  pdbRecordType->papFldDes[40]->size=sizeof(prec->udf);
  pdbRecordType->papFldDes[40]->offset=(short)((char *)&prec->udf - (char *)prec);
  pdbRecordType->papFldDes[41]->size=sizeof(prec->time);
  pdbRecordType->papFldDes[41]->offset=(short)((char *)&prec->time - (char *)prec);
  pdbRecordType->papFldDes[42]->size=sizeof(prec->flnk);
  pdbRecordType->papFldDes[42]->offset=(short)((char *)&prec->flnk - (char *)prec);
  pdbRecordType->papFldDes[43]->size=sizeof(prec->val);
  pdbRecordType->papFldDes[43]->offset=(short)((char *)&prec->val - (char *)prec);
  pdbRecordType->papFldDes[44]->size=sizeof(prec->oval);
  pdbRecordType->papFldDes[44]->offset=(short)((char *)&prec->oval - (char *)prec);
  pdbRecordType->papFldDes[45]->size=sizeof(prec->dcmt);
  pdbRecordType->papFldDes[45]->offset=(short)((char *)&prec->dcmt - (char *)prec);
  pdbRecordType->papFldDes[46]->size=sizeof(prec->nt);
  pdbRecordType->papFldDes[46]->offset=(short)((char *)&prec->nt - (char *)prec);
  pdbRecordType->papFldDes[47]->size=sizeof(prec->toff);
  pdbRecordType->papFldDes[47]->offset=(short)((char *)&prec->toff - (char *)prec);
  pdbRecordType->papFldDes[48]->size=sizeof(prec->dly);
  pdbRecordType->papFldDes[48]->offset=(short)((char *)&prec->dly - (char *)prec);
  pdbRecordType->papFldDes[49]->size=sizeof(prec->loca);
  pdbRecordType->papFldDes[49]->offset=(short)((char *)&prec->loca - (char *)prec);
  pdbRecordType->papFldDes[50]->size=sizeof(prec->link);
  pdbRecordType->papFldDes[50]->offset=(short)((char *)&prec->link - (char *)prec);
  pdbRecordType->papFldDes[51]->size=sizeof(prec->rack);
  pdbRecordType->papFldDes[51]->offset=(short)((char *)&prec->rack - (char *)prec);
  pdbRecordType->papFldDes[52]->size=sizeof(prec->slot);
  pdbRecordType->papFldDes[52]->offset=(short)((char *)&prec->slot - (char *)prec);
  pdbRecordType->papFldDes[53]->size=sizeof(prec->mcw);
  pdbRecordType->papFldDes[53]->offset=(short)((char *)&prec->mcw - (char *)prec);
  pdbRecordType->papFldDes[54]->size=sizeof(prec->t0);
  pdbRecordType->papFldDes[54]->offset=(short)((char *)&prec->t0 - (char *)prec);
  pdbRecordType->papFldDes[55]->size=sizeof(prec->t1);
  pdbRecordType->papFldDes[55]->offset=(short)((char *)&prec->t1 - (char *)prec);
  pdbRecordType->papFldDes[56]->size=sizeof(prec->t2);
  pdbRecordType->papFldDes[56]->offset=(short)((char *)&prec->t2 - (char *)prec);
  pdbRecordType->papFldDes[57]->size=sizeof(prec->t3);
  pdbRecordType->papFldDes[57]->offset=(short)((char *)&prec->t3 - (char *)prec);
  pdbRecordType->papFldDes[58]->size=sizeof(prec->t4);
  pdbRecordType->papFldDes[58]->offset=(short)((char *)&prec->t4 - (char *)prec);
  pdbRecordType->papFldDes[59]->size=sizeof(prec->t5);
  pdbRecordType->papFldDes[59]->offset=(short)((char *)&prec->t5 - (char *)prec);
  pdbRecordType->papFldDes[60]->size=sizeof(prec->t6);
  pdbRecordType->papFldDes[60]->offset=(short)((char *)&prec->t6 - (char *)prec);
  pdbRecordType->papFldDes[61]->size=sizeof(prec->t7);
  pdbRecordType->papFldDes[61]->offset=(short)((char *)&prec->t7 - (char *)prec);
  pdbRecordType->papFldDes[62]->size=sizeof(prec->t8);
  pdbRecordType->papFldDes[62]->offset=(short)((char *)&prec->t8 - (char *)prec);
  pdbRecordType->papFldDes[63]->size=sizeof(prec->t9);
  pdbRecordType->papFldDes[63]->offset=(short)((char *)&prec->t9 - (char *)prec);
    pdbRecordType->rec_size = sizeof(*prec);
    return(0);
}
epicsExportRegistrar(abDcmRecordSizeOffset);
#ifdef __cplusplus
}
#endif
#endif /*GEN_SIZE_OFFSET*/
